Module: JSON

Defined in:
lib/gistto/enhancements/json.rb,
lib/gistto/enhancements/json.rb,
lib/gistto/enhancements/json.rb

Defined Under Namespace

Modules: Pure Classes: CircularDatastructure, GeneratorError, JSONError, MissingUnicodeSupport, NestingError, ParserError

Constant Summary collapse

MAP =
{
  "\x0" => '\u0000',
  "\x1" => '\u0001',
  "\x2" => '\u0002',
  "\x3" => '\u0003',
  "\x4" => '\u0004',
  "\x5" => '\u0005',
  "\x6" => '\u0006',
  "\x7" => '\u0007',
  "\b"  =>  '\b',
  "\t"  =>  '\t',
  "\n"  =>  '\n',
  "\xb" => '\u000b',
  "\f"  =>  '\f',
  "\r"  =>  '\r',
  "\xe" => '\u000e',
  "\xf" => '\u000f',
  "\x10" => '\u0010',
  "\x11" => '\u0011',
  "\x12" => '\u0012',
  "\x13" => '\u0013',
  "\x14" => '\u0014',
  "\x15" => '\u0015',
  "\x16" => '\u0016',
  "\x17" => '\u0017',
  "\x18" => '\u0018',
  "\x19" => '\u0019',
  "\x1a" => '\u001a',
  "\x1b" => '\u001b',
  "\x1c" => '\u001c',
  "\x1d" => '\u001d',
  "\x1e" => '\u001e',
  "\x1f" => '\u001f',
  '"'   =>  '\"',
  '\\'  =>  '\\\\',
}
NaN =
0.0/0
Infinity =
1.0/0
MinusInfinity =
-Infinity
UnparserError =

For backwards compatibility

GeneratorError

Class Attribute Summary collapse

Class Method Summary collapse

Class Attribute Details

.create_idObject

This is create identifier, which is used to decide if the json_create hook of a class should be called. It defaults to ‘json_class’.



925
926
927
# File 'lib/gistto/enhancements/json.rb', line 925

def create_id
  @create_id
end

.dump_default_optionsObject

The global default options for the JSON.dump method:

:max_nesting: false
:allow_nan:   true
:quirks_mode: true


1178
1179
1180
# File 'lib/gistto/enhancements/json.rb', line 1178

def dump_default_options
  @dump_default_options
end

.generatorObject

Returns the JSON generator module that is used by JSON. This is either JSON::Ext::Generator or JSON::Pure::Generator.



917
918
919
# File 'lib/gistto/enhancements/json.rb', line 917

def generator
  @generator
end

.load_default_optionsObject

The global default options for the JSON.load method:

:max_nesting: false
:allow_nan:   true
:quirks_mode: true


1123
1124
1125
# File 'lib/gistto/enhancements/json.rb', line 1123

def load_default_options
  @load_default_options
end

.parserObject

Returns the JSON parser class that is used by JSON. This is either JSON::Ext::Parser or JSON::Pure::Parser.



852
853
854
# File 'lib/gistto/enhancements/json.rb', line 852

def parser
  @parser
end

.stateObject

Returns the JSON generator state class that is used by JSON. This is either JSON::Ext::Generator::State or JSON::Pure::Generator::State.



921
922
923
# File 'lib/gistto/enhancements/json.rb', line 921

def state
  @state
end

Class Method Details

.[](object, opts = {}) ⇒ Object

If object is string-like, parse the string and return the parsed result as a Ruby data structure. Otherwise generate a JSON text from the Ruby data structure object and return it.

The opts argument is passed through to generate/parse respectively. See generate and parse for their documentation.



842
843
844
845
846
847
848
# File 'lib/gistto/enhancements/json.rb', line 842

def [](object, opts = {})
  if object.respond_to? :to_str
    JSON.parse(object.to_str, opts)
  else
    JSON.generate(object, opts)
  end
end

.const_defined_in?(modul, constant) ⇒ Boolean

Returns:

  • (Boolean)


1246
1247
1248
# File 'lib/gistto/enhancements/json.rb', line 1246

def self.const_defined_in?(modul, constant)
  modul.const_defined?(constant)
end

.deep_const_get(path) ⇒ Object

Return the constant located at path. The format of path has to be either ::A::B::C or A::B::C. In any case, A has to be located at the top level (absolute namespace path?). If there doesn’t exist a constant at the given path, an ArgumentError is raised.



865
866
867
868
869
870
871
872
873
874
875
876
877
878
# File 'lib/gistto/enhancements/json.rb', line 865

def deep_const_get(path) # :nodoc:
  path.to_s.split(/::/).inject(Object) do |p, c|
    case
    when c.empty?                     then p
    when JSON.const_defined_in?(p, c) then p.const_get(c)
    else
      begin
        p.const_missing(c)
      rescue NameError => e
        raise ArgumentError, "can't get const #{path}: #{e}"
      end
    end
  end
end

.dump(obj, anIO = nil, limit = nil) ⇒ Object

Dumps obj as a JSON string, i.e. calls generate on the object and returns the result.

If anIO (an IO-like object or an object that responds to the write method) was given, the resulting JSON is written to it.

If the number of nested arrays or objects exceeds limit, an ArgumentError exception is raised. This argument is similar (but not exactly the same!) to the limit argument in Marshal.dump.

The default options for the generator can be changed via the dump_default_options method.

This method is part of the implementation of the load/dump interface of Marshal and YAML.



1201
1202
1203
1204
1205
1206
1207
1208
1209
1210
1211
1212
1213
1214
1215
1216
1217
1218
1219
1220
# File 'lib/gistto/enhancements/json.rb', line 1201

def dump(obj, anIO = nil, limit = nil)
  if anIO and limit.nil?
    anIO = anIO.to_io if anIO.respond_to?(:to_io)
    unless anIO.respond_to?(:write)
      limit = anIO
      anIO = nil
    end
  end
  opts = JSON.dump_default_options
  limit and opts.update(:max_nesting => limit)
  result = generate(obj, opts)
  if anIO
    anIO.write result
    anIO
  else
    result
  end
rescue JSON::NestingError
  raise ArgumentError, "exceed depth limit"
end

.fast_generate(obj, opts = nil) ⇒ Object Also known as: fast_unparse

Generate a JSON document from the Ruby data structure obj and return it. This method disables the checks for circles in Ruby objects.

WARNING: Be careful not to pass any Ruby data structures with circles as obj argument because this will cause JSON to go into an infinite loop.



1062
1063
1064
1065
1066
1067
1068
1069
1070
1071
1072
1073
1074
1075
1076
1077
1078
1079
# File 'lib/gistto/enhancements/json.rb', line 1062

def fast_generate(obj, opts = nil)
  if State === opts
    state, opts = opts, nil
  else
    state = FAST_STATE_PROTOTYPE.dup
  end
  if opts
    if opts.respond_to? :to_hash
      opts = opts.to_hash
    elsif opts.respond_to? :to_h
      opts = opts.to_h
    else
      raise TypeError, "can't convert #{opts.class} into Hash"
    end
    state.configure(opts)
  end
  state.generate(obj)
end

.generate(obj, opts = nil) ⇒ Object Also known as: unparse

Generate a JSON document from the Ruby data structure obj and return it. state is * a JSON::State object,

  • or a Hash like object (responding to to_hash),

  • an object convertible into a hash by a to_h method,

that is used as or to configure a State object.

It defaults to a state object, that creates the shortest possible JSON text in one line, checks for circular data structures and doesn’t allow NaN, Infinity, and -Infinity.

A state hash can have the following keys:

  • indent: a string used to indent levels (default: ”),

  • space: a string that is put after, a : or , delimiter (default: ”),

  • space_before: a string that is put before a : pair delimiter (default: ”),

  • object_nl: a string that is put at the end of a JSON object (default: ”),

  • array_nl: a string that is put at the end of a JSON array (default: ”),

  • allow_nan: true if NaN, Infinity, and -Infinity should be generated, otherwise an exception is thrown if these values are encountered. This options defaults to false.

  • max_nesting: The maximum depth of nesting allowed in the data structures from which JSON is to be generated. Disable depth checking with :max_nesting => false, it defaults to 19.

See also the fast_generate for the fastest creation method with the least amount of sanity checks, and the pretty_generate method for some defaults for pretty output.



1031
1032
1033
1034
1035
1036
1037
1038
1039
1040
1041
1042
1043
1044
1045
1046
1047
1048
# File 'lib/gistto/enhancements/json.rb', line 1031

def generate(obj, opts = nil)
  if State === opts
    state, opts = opts, nil
  else
    state = SAFE_STATE_PROTOTYPE.dup
  end
  if opts
    if opts.respond_to? :to_hash
      opts = opts.to_hash
    elsif opts.respond_to? :to_h
      opts = opts.to_h
    else
      raise TypeError, "can't convert #{opts.class} into Hash"
    end
    state = state.configure(opts)
  end
  state.generate(obj)
end

.iconv(to, from, string) ⇒ Object

Encodes string using iconv library



1234
1235
1236
# File 'lib/gistto/enhancements/json.rb', line 1234

def self.iconv(to, from, string)
  string.encode(to, from)
end

.load(source, proc = nil) ⇒ Object Also known as: restore

Load a ruby data structure from a JSON source and return it. A source can either be a string-like object, an IO-like object, or an object responding to the read method. If proc was given, it will be called with any nested Ruby object as an argument recursively in depth first order. The default options for the parser can be changed via the load_default_options method.

This method is part of the implementation of the load/dump interface of Marshal and YAML.



1139
1140
1141
1142
1143
1144
1145
1146
1147
1148
1149
1150
1151
1152
1153
1154
# File 'lib/gistto/enhancements/json.rb', line 1139

def load(source, proc = nil)
  opts = load_default_options
  if source.respond_to? :to_str
    source = source.to_str
  elsif source.respond_to? :to_io
    source = source.to_io.read
  elsif source.respond_to?(:read)
    source = source.read
  end
  if opts[:quirks_mode] && (source.nil? || source.empty?)
    source = 'null'
  end
  result = parse(source, opts)
  recurse_proc(result, &proc) if proc
  result
end

.parse(source, opts = {}) ⇒ Object

Parse the JSON document source into a Ruby data structure and return it.

opts can have the following keys:

  • max_nesting: The maximum depth of nesting allowed in the parsed data structures. Disable depth checking with :max_nesting => false. It defaults to 19.

  • allow_nan: If set to true, allow NaN, Infinity and -Infinity in defiance of RFC 4627 to be parsed by the Parser. This option defaults to false.

  • symbolize_names: If set to true, returns symbols for the names (keys) in a JSON object. Otherwise strings are returned. Strings are the default.

  • create_additions: If set to false, the Parser doesn’t create additions even if a matching class and create_id was found. This option defaults to true.

  • object_class: Defaults to Hash

  • array_class: Defaults to Array



978
979
980
# File 'lib/gistto/enhancements/json.rb', line 978

def parse(source, opts = {})
  Parser.new(source, opts).parse
end

.parse!(source, opts = {}) ⇒ Object

Parse the JSON document source into a Ruby data structure and return it. The bang version of the parse method defaults to the more dangerous values for the opts hash, so be sure only to parse trusted source documents.

opts can have the following keys:

  • max_nesting: The maximum depth of nesting allowed in the parsed data structures. Enable depth checking with :max_nesting => anInteger. The parse! methods defaults to not doing max depth checking: This can be dangerous if someone wants to fill up your stack.

  • allow_nan: If set to true, allow NaN, Infinity, and -Infinity in defiance of RFC 4627 to be parsed by the Parser. This option defaults to true.

  • create_additions: If set to false, the Parser doesn’t create additions even if a matching class and create_id was found. This option defaults to true.



997
998
999
1000
1001
1002
1003
# File 'lib/gistto/enhancements/json.rb', line 997

def parse!(source, opts = {})
  opts = {
    :max_nesting  => false,
    :allow_nan    => true
  }.update(opts)
  Parser.new(source, opts).parse
end

.pretty_generate(obj, opts = nil) ⇒ Object Also known as: pretty_unparse

Generate a JSON document from the Ruby data structure obj and return it. The returned document is a prettier form of the document returned by #unparse.

The opts argument can be used to configure the generator. See the generate method for a more detailed explanation.



1093
1094
1095
1096
1097
1098
1099
1100
1101
1102
1103
1104
1105
1106
1107
1108
1109
1110
# File 'lib/gistto/enhancements/json.rb', line 1093

def pretty_generate(obj, opts = nil)
  if State === opts
    state, opts = opts, nil
  else
    state = PRETTY_STATE_PROTOTYPE.dup
  end
  if opts
    if opts.respond_to? :to_hash
      opts = opts.to_hash
    elsif opts.respond_to? :to_h
      opts = opts.to_h
    else
      raise TypeError, "can't convert #{opts.class} into Hash"
    end
    state.configure(opts)
  end
  state.generate(obj)
end

.recurse_proc(result, &proc) ⇒ Object

Recursively calls passed Proc if the parsed data structure is an Array or Hash



1157
1158
1159
1160
1161
1162
1163
1164
1165
1166
1167
1168
# File 'lib/gistto/enhancements/json.rb', line 1157

def recurse_proc(result, &proc)
  case result
  when Array
    result.each { |x| recurse_proc x, &proc }
    proc.call result
  when Hash
    result.each { |x, y| recurse_proc x, &proc; recurse_proc y, &proc }
    proc.call result
  else
    proc.call result
  end
end

.swap!(string) ⇒ Object

Swap consecutive bytes of string in place.



1223
1224
1225
1226
1227
1228
1229
# File 'lib/gistto/enhancements/json.rb', line 1223

def self.swap!(string) # :nodoc:
  0.upto(string.size / 2) do |i|
    break unless string[2 * i + 1]
    string[2 * i], string[2 * i + 1] = string[2 * i + 1], string[2 * i]
  end
  string
end

.utf8_to_json(string) ⇒ Object

:nodoc:



402
403
404
405
406
407
408
409
# File 'lib/gistto/enhancements/json.rb', line 402

def utf8_to_json(string) # :nodoc:
  string = string.dup
  string << '' # XXX workaround: avoid buffer sharing
  string.force_encoding(::Encoding::ASCII_8BIT)
  string.gsub!(/["\\\x0-\x1f]/) { MAP[$&] }
  string.force_encoding(::Encoding::UTF_8)
  string
end

.utf8_to_json_ascii(string) ⇒ Object

:nodoc:



411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
# File 'lib/gistto/enhancements/json.rb', line 411

def utf8_to_json_ascii(string) # :nodoc:
  string = string.dup
  string << '' # XXX workaround: avoid buffer sharing
  string.force_encoding(::Encoding::ASCII_8BIT)
  string.gsub!(/["\\\x0-\x1f]/) { MAP[$&] }
  string.gsub!(/(
                  (?:
                    [\xc2-\xdf][\x80-\xbf]    |
                    [\xe0-\xef][\x80-\xbf]{2} |
                    [\xf0-\xf4][\x80-\xbf]{3}
                  )+ |
                  [\x80-\xc1\xf5-\xff]       # invalid
                )/nx) { |c|
                  c.size == 1 and raise GeneratorError, "invalid utf8 byte: '#{c}'"
                  s = JSON.iconv('utf-16be', 'utf-8', c).unpack('H*')[0]
                  s.gsub!(/.{4}/n, '\\\\u\&')
                }
  string.force_encoding(::Encoding::UTF_8)
  string
rescue => e
  raise GeneratorError, "Caught #{e.class}: #{e}"
end